@@ -8,19 +8,34 @@ use crate::payment::Payment;
 use crate::payment::PaymentProcessor;
 use crate::shipping::FulfillmentTypeSelection;
 
+pub async fn get_handler<C: Context + Send>(
+    ctx: &mut C,
+    id: &str,
+) -> Result<
+    Checkout<Payment<<C as PaymentProcessor>::Data, <C as PaymentProcessor>::ProcessArgs>>,
+    Error,
+> {
+    let maybe_co = ctx.get_checkout(id).await?;
+
+    if maybe_co.is_none() {
+        return Err(new_not_found_error(
+            "the requested checkout could not be found",
+        ));
+    }
+
+    let mut co = maybe_co.unwrap();
+    co.populate_associations(ctx).await?;
+
+    Ok(co)
+}
+
 pub async fn create_handler<C: Context + Send>(
     ctx: &mut C,
     currency: Currency,
     contact: Option<Contact>,
     shipping_address: Option<Address>,
 ) -> Result<
-    Checkout<
-        Payment<
-            <C as PaymentProcessor>::PrivateData,
-            <C as PaymentProcessor>::PublicData,
-            <C as PaymentProcessor>::ProcessArgs,
-        >,
-    >,
+    Checkout<Payment<<C as PaymentProcessor>::Data, <C as PaymentProcessor>::ProcessArgs>>,
     Error,
 > {
     let mut co = Checkout::new(ctx, currency, contact, shipping_address).await?;
@@ -35,13 +50,7 @@ pub async fn add_item_handler<C: Context + Send>(
     sku: String,
     quantity: u64,
 ) -> Result<
-    Checkout<
-        Payment<
-            <C as PaymentProcessor>::PrivateData,
-            <C as PaymentProcessor>::PublicData,
-            <C as PaymentProcessor>::ProcessArgs,
-        >,
-    >,
+    Checkout<Payment<<C as PaymentProcessor>::Data, <C as PaymentProcessor>::ProcessArgs>>,
     Error,
 > {
     let maybe_co = ctx.get_checkout_for_update(&checkout_id).await?;
@@ -65,13 +74,7 @@ pub async fn remove_item_handler<C: Context + Send>(
     sku: String,
     quantity: u64,
 ) -> Result<
-    Checkout<
-        Payment<
-            <C as PaymentProcessor>::PrivateData,
-            <C as PaymentProcessor>::PublicData,
-            <C as PaymentProcessor>::ProcessArgs,
-        >,
-    >,
+    Checkout<Payment<<C as PaymentProcessor>::Data, <C as PaymentProcessor>::ProcessArgs>>,
     Error,
 > {
     let maybe_co = ctx.get_checkout_for_update(&checkout_id).await?;
@@ -93,13 +96,7 @@ pub async fn confirm_items_handler<C: Context + Send>(
     ctx: &mut C,
     checkout_id: &str,
 ) -> Result<
-    Checkout<
-        Payment<
-            <C as PaymentProcessor>::PrivateData,
-            <C as PaymentProcessor>::PublicData,
-            <C as PaymentProcessor>::ProcessArgs,
-        >,
-    >,
+    Checkout<Payment<<C as PaymentProcessor>::Data, <C as PaymentProcessor>::ProcessArgs>>,
     Error,
 > {
     let maybe_co = ctx.get_checkout_for_update(&checkout_id).await?;
@@ -122,13 +119,7 @@ pub async fn update_contact_handler<C: Context + Send>(
     checkout_id: &str,
     contact: Contact,
 ) -> Result<
-    Checkout<
-        Payment<
-            <C as PaymentProcessor>::PrivateData,
-            <C as PaymentProcessor>::PublicData,
-            <C as PaymentProcessor>::ProcessArgs,
-        >,
-    >,
+    Checkout<Payment<<C as PaymentProcessor>::Data, <C as PaymentProcessor>::ProcessArgs>>,
     Error,
 > {
     let maybe_co = ctx.get_checkout_for_update(&checkout_id).await?;
@@ -151,13 +142,7 @@ pub async fn update_shipping_address_handler<C: Context + Send>(
     checkout_id: &str,
     shipping_address: Address,
 ) -> Result<
-    Checkout<
-        Payment<
-            <C as PaymentProcessor>::PrivateData,
-            <C as PaymentProcessor>::PublicData,
-            <C as PaymentProcessor>::ProcessArgs,
-        >,
-    >,
+    Checkout<Payment<<C as PaymentProcessor>::Data, <C as PaymentProcessor>::ProcessArgs>>,
     Error,
 > {
     let maybe_co = ctx.get_checkout_for_update(&checkout_id).await?;
@@ -180,13 +165,7 @@ pub async fn update_fulfillment_type_handler<C: Context + Send>(
     checkout_id: &str,
     fulfillment_type: FulfillmentTypeSelection,
 ) -> Result<
-    Checkout<
-        Payment<
-            <C as PaymentProcessor>::PrivateData,
-            <C as PaymentProcessor>::PublicData,
-            <C as PaymentProcessor>::ProcessArgs,
-        >,
-    >,
+    Checkout<Payment<<C as PaymentProcessor>::Data, <C as PaymentProcessor>::ProcessArgs>>,
     Error,
 > {
     let maybe_co = ctx.get_checkout_for_update(&checkout_id).await?;
@@ -209,13 +188,7 @@ pub async fn initiate_payment_handler<C: Context + Send>(
     checkout_id: &str,
     args: <C as PaymentProcessor>::ProcessArgs,
 ) -> Result<
-    Checkout<
-        Payment<
-            <C as PaymentProcessor>::PrivateData,
-            <C as PaymentProcessor>::PublicData,
-            <C as PaymentProcessor>::ProcessArgs,
-        >,
-    >,
+    Checkout<Payment<<C as PaymentProcessor>::Data, <C as PaymentProcessor>::ProcessArgs>>,
     Error,
 > {
     let maybe_co = ctx.get_checkout_for_update(&checkout_id).await?;
@@ -239,13 +212,7 @@ pub async fn initiate_order_handler<C: Context + Send>(
     checkout_id: &str,
     args: <C as PaymentProcessor>::ProcessArgs,
 ) -> Result<
-    Checkout<
-        Payment<
-            <C as PaymentProcessor>::PrivateData,
-            <C as PaymentProcessor>::PublicData,
-            <C as PaymentProcessor>::ProcessArgs,
-        >,
-    >,
+    Checkout<Payment<<C as PaymentProcessor>::Data, <C as PaymentProcessor>::ProcessArgs>>,
     Error,
 > {
     let maybe_co = ctx.get_checkout_for_update(&checkout_id).await?;
