@@ -14,18 +14,33 @@ use super::LookupKey;
 use super::Order;
 use super::OrderState;
 
+pub async fn get_handler<C: Context + Send>(
+    ctx: &mut C,
+    id: &str,
+) -> Result<
+    Order<Payment<<C as PaymentProcessor>::Data, <C as PaymentProcessor>::ProcessArgs>>,
+    Error,
+> {
+    let maybe_od = ctx.get_order(id).await?;
+
+    if maybe_od.is_none() {
+        return Err(new_not_found_error(
+            "the requested order could not be found",
+        ));
+    }
+
+    let mut od = maybe_od.unwrap();
+    od.populate_associations(ctx).await?;
+
+    Ok(od)
+}
+
 pub async fn confirm_handler<C: Context + Send>(
     internal_ctx: &InternalContext,
     ctx: &mut C,
     order_id: &str,
 ) -> Result<
-    Order<
-        Payment<
-            <C as PaymentProcessor>::PrivateData,
-            <C as PaymentProcessor>::PublicData,
-            <C as PaymentProcessor>::ProcessArgs,
-        >,
-    >,
+    Order<Payment<<C as PaymentProcessor>::Data, <C as PaymentProcessor>::ProcessArgs>>,
     Error,
 > {
     let maybe_od = ctx.get_order_for_update(&order_id).await?;
@@ -47,13 +62,7 @@ pub async fn cancel_handler<C: Context + Send>(
     ctx: &mut C,
     order_id: &str,
 ) -> Result<
-    Order<
-        Payment<
-            <C as PaymentProcessor>::PrivateData,
-            <C as PaymentProcessor>::PublicData,
-            <C as PaymentProcessor>::ProcessArgs,
-        >,
-    >,
+    Order<Payment<<C as PaymentProcessor>::Data, <C as PaymentProcessor>::ProcessArgs>>,
     Error,
 > {
     // first retrieve the associated payment for update to avoid deadlock
@@ -91,13 +100,7 @@ pub async fn reconcile_handler<C: Context + Send>(
     lookup_key: LookupKey,
 ) -> Result<(), Error> {
     let maybe_order: Option<
-        Order<
-            Payment<
-                <C as PaymentProcessor>::PrivateData,
-                <C as PaymentProcessor>::PublicData,
-                <C as PaymentProcessor>::ProcessArgs,
-            >,
-        >,
+        Order<Payment<<C as PaymentProcessor>::Data, <C as PaymentProcessor>::ProcessArgs>>,
     > = match &lookup_key {
         LookupKey::CheckoutId(checkout_id) => get_order_by_checkout_id(ctx, checkout_id).await?,
         LookupKey::OrderId(order_id) => ctx.get_order_for_update(order_id).await?,
@@ -119,25 +122,11 @@ async fn get_order_by_checkout_id<C: Context + Send>(
     ctx: &mut C,
     checkout_id: &str,
 ) -> Result<
-    Option<
-        Order<
-            Payment<
-                <C as PaymentProcessor>::PrivateData,
-                <C as PaymentProcessor>::PublicData,
-                <C as PaymentProcessor>::ProcessArgs,
-            >,
-        >,
-    >,
+    Option<Order<Payment<<C as PaymentProcessor>::Data, <C as PaymentProcessor>::ProcessArgs>>>,
     Error,
 > {
     let maybe_checkout: Option<
-        Checkout<
-            Payment<
-                <C as PaymentProcessor>::PrivateData,
-                <C as PaymentProcessor>::PublicData,
-                <C as PaymentProcessor>::ProcessArgs,
-            >,
-        >,
+        Checkout<Payment<<C as PaymentProcessor>::Data, <C as PaymentProcessor>::ProcessArgs>>,
     > = ctx.get_checkout_for_update(checkout_id).await?;
     if maybe_checkout.is_none() {
         return Err(new_not_found_error(
@@ -159,20 +148,13 @@ async fn get_order_payment_for_update<C: Context + Send>(
     ctx: &mut C,
     order_id: &str,
 ) -> Result<
-    Option<
-        Payment<
-            <C as PaymentProcessor>::PrivateData,
-            <C as PaymentProcessor>::PublicData,
-            <C as PaymentProcessor>::ProcessArgs,
-        >,
-    >,
+    Option<Payment<<C as PaymentProcessor>::Data, <C as PaymentProcessor>::ProcessArgs>>,
     Error,
 > {
     let maybe_od: Option<
         Order<
             Payment<
-                <C as PaymentProcessor>::PrivateData,
-                <C as PaymentProcessor>::PublicData,
+                <C as PaymentProcessor>::Data,
                 <C as PaymentProcessor>::ProcessArgs,
             >,
         >,
