@@ -14,12 +14,13 @@ use crate::order::Order;
 pub mod handlers;
 
 #[derive(Serialize, Deserialize, JsonSchema)]
-pub struct Payment<PrivateData, PublicData, ProcessArgs> {
+pub struct Payment<Data, ProcessArgs> {
     pub id: String,
     pub state: PaymentState<ProcessArgs>,
     pub initiator_id: InitiatorId,
     pub charge_amount: Money,
-    pub data: PaymentData<PrivateData, PublicData>,
+    pub correlation_id: Option<String>,
+    pub data: Option<Data>,
 }
 
 #[derive(Serialize, Deserialize, JsonSchema)]
@@ -43,32 +44,30 @@ pub enum Initiator<'a, P: Sync + Send> {
 
 #[async_trait]
 pub trait PaymentProcessor {
+    type Data: Sync + Send + Serialize + DeserializeOwned;
     type ProcessArgs: Sync + Send + Serialize + DeserializeOwned;
-    type PrivateData: Sync + Send + Serialize + DeserializeOwned;
-    type PublicData: Sync + Send + Serialize + DeserializeOwned;
 
     async fn initiate_payment(
         &mut self,
-        _payment: &Payment<Self::PrivateData, Self::PublicData, Self::ProcessArgs>,
+        _payment: &Payment<Self::Data, Self::ProcessArgs>,
         _args: &Self::ProcessArgs,
-    ) -> Result<PaymentData<Self::PrivateData, Self::PublicData>, Error> {
+    ) -> Result<PaymentData<Self::Data>, Error> {
         Ok(PaymentData {
             correlation_id: None,
-            private_data: None,
-            public_data: None,
+            data: None,
         })
     }
 
     async fn process_payment(
         &mut self,
-        payment: &Payment<Self::PrivateData, Self::PublicData, Self::ProcessArgs>,
+        payment: &Payment<Self::Data, Self::ProcessArgs>,
         args: &Self::ProcessArgs,
-    ) -> Result<ProcessingResult<Self::PrivateData, Self::PublicData>, Error>;
+    ) -> Result<ProcessingResult<Self::Data>, Error>;
 
     // this is called synchronously and only once per revert request
     async fn cancel_payment(
         &mut self,
-        payment: &Payment<Self::PrivateData, Self::PublicData, Self::ProcessArgs>,
+        payment: &Payment<Self::Data, Self::ProcessArgs>,
     ) -> Result<CancellationResult, Error>;
 }
 
@@ -77,8 +76,7 @@ pub trait PaymentStore: PaymentProcessor {
     async fn create_payment(
         &mut self,
         payment: &Payment<
-            <Self as PaymentProcessor>::PrivateData,
-            <Self as PaymentProcessor>::PublicData,
+            <Self as PaymentProcessor>::Data,
             <Self as PaymentProcessor>::ProcessArgs,
         >,
     ) -> Result<(), Error>;
@@ -86,8 +84,7 @@ pub trait PaymentStore: PaymentProcessor {
     async fn update_payment(
         &mut self,
         payment: &Payment<
-            <Self as PaymentProcessor>::PrivateData,
-            <Self as PaymentProcessor>::PublicData,
+            <Self as PaymentProcessor>::Data,
             <Self as PaymentProcessor>::ProcessArgs,
         >,
     ) -> Result<(), Error>;
@@ -112,9 +109,9 @@ pub trait PaymentStore: PaymentProcessor {
     }
 }
 
-pub struct ProcessingResult<PrivateData, PublicData> {
+pub struct ProcessingResult<Data> {
     pub state: InternalPaymentState,
-    pub data: PaymentData<PrivateData, PublicData>,
+    pub payment_data: PaymentData<Data>,
 }
 
 pub struct CancellationResult {
@@ -122,10 +119,9 @@ pub struct CancellationResult {
 }
 
 #[derive(Serialize, Deserialize, JsonSchema)]
-pub struct PaymentData<PrivateData, PublicData> {
+pub struct PaymentData<Data> {
     pub correlation_id: Option<String>,
-    pub private_data: Option<PrivateData>,
-    pub public_data: Option<PublicData>,
+    pub data: Option<Data>,
 }
 
 #[derive(Serialize, Deserialize, JsonSchema)]
@@ -142,14 +138,7 @@ async fn new<'a, C: Context + Send, P: Sync + Send + Serialize + DeserializeOwne
     initiator: Initiator<'a, P>,
     charge_amount: Money,
     args: <C as PaymentProcessor>::ProcessArgs,
-) -> Result<
-    Payment<
-        <C as PaymentProcessor>::PrivateData,
-        <C as PaymentProcessor>::PublicData,
-        <C as PaymentProcessor>::ProcessArgs,
-    >,
-    Error,
-> {
+) -> Result<Payment<<C as PaymentProcessor>::Data, <C as PaymentProcessor>::ProcessArgs>, Error> {
     let mut payment = Payment {
         id: Uuid::new_v4().to_string(),
         state: PaymentState::Idle,
@@ -158,16 +147,16 @@ async fn new<'a, C: Context + Send, P: Sync + Send + Serialize + DeserializeOwne
             Initiator::Order(od) => InitiatorId::Order(od.id.clone()),
         },
         charge_amount,
-        data: PaymentData {
-            correlation_id: None,
-            private_data: None,
-            public_data: None,
-        },
+        correlation_id: None,
+        data: None,
     };
 
-    payment.data = ctx.initiate_payment(&payment, &args).await?;
-    ctx.create_payment(&payment).await?;
+    let payment_data = ctx.initiate_payment(&payment, &args).await?;
 
+    payment.correlation_id = payment_data.correlation_id;
+    payment.data = payment_data.data;
+
+    ctx.create_payment(&payment).await?;
     Ok(payment)
 }
 
@@ -175,11 +164,7 @@ async fn new<'a, C: Context + Send, P: Sync + Send + Serialize + DeserializeOwne
 pub async fn queue_for_processing<C: Context + Send>(
     internal_ctx: &InternalContext,
     ctx: &mut C,
-    payment: &mut Payment<
-        <C as PaymentProcessor>::PrivateData,
-        <C as PaymentProcessor>::PublicData,
-        <C as PaymentProcessor>::ProcessArgs,
-    >,
+    payment: &mut Payment<<C as PaymentProcessor>::Data, <C as PaymentProcessor>::ProcessArgs>,
     args: <C as PaymentProcessor>::ProcessArgs,
 ) -> Result<(), Error> {
     match &payment.state {
@@ -215,17 +200,16 @@ pub async fn queue_for_processing<C: Context + Send>(
 pub async fn process<'a, C: Context + Send, P: Sync + Send + Serialize + DeserializeOwned>(
     internal_ctx: &InternalContext,
     ctx: &mut C,
-    payment: &mut Payment<
-        <C as PaymentProcessor>::PrivateData,
-        <C as PaymentProcessor>::PublicData,
-        <C as PaymentProcessor>::ProcessArgs,
-    >,
+    payment: &mut Payment<<C as PaymentProcessor>::Data, <C as PaymentProcessor>::ProcessArgs>,
     initiator: Initiator<'a, P>,
 ) -> Result<(), Error> {
     match &payment.state {
         PaymentState::Processing(args) => {
             let result = ctx.process_payment(payment, args).await?;
-            payment.data = result.data;
+
+            payment.correlation_id = result.payment_data.correlation_id;
+            payment.data = result.payment_data.data;
+
             match result.state {
                 InternalPaymentState::Incomplete | InternalPaymentState::Failed => {
                     payment.state = PaymentState::Idle
@@ -255,11 +239,7 @@ pub async fn process<'a, C: Context + Send, P: Sync + Send + Serialize + Deseria
 pub async fn cancel<'a, C: Context + Send, P: Sync + Send + Serialize + DeserializeOwned>(
     internal_ctx: &InternalContext,
     ctx: &mut C,
-    payment: &mut Payment<
-        <C as PaymentProcessor>::PrivateData,
-        <C as PaymentProcessor>::PublicData,
-        <C as PaymentProcessor>::ProcessArgs,
-    >,
+    payment: &mut Payment<<C as PaymentProcessor>::Data, <C as PaymentProcessor>::ProcessArgs>,
     initiator: Initiator<'a, P>,
 ) -> Result<(), Error> {
     match &payment.state {
